@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # 2011 Steven Armstrong (steven-cdist at armstrong.cc)
-# 2011-2012 Nico Schottelius (nico-cdist at schottelius.org)
+# 2011-2013 Nico Schottelius (nico-cdist at schottelius.org)
 #
 # This file is part of cdist.
 #
@@ -27,6 +27,7 @@ import re
 import subprocess
 import shutil
 import logging
+import tempfile
 
 import cdist
 from cdist import core
@@ -38,20 +39,35 @@ class Local(object):
     Directly accessing the local side from python code is a bug.
 
     """
-    def __init__(self, target_host, out_path, exec_path, add_conf_dirs=None, cache_dir=None):
+    def __init__(self,
+                 target_host,
+                 exec_path=sys.argv[0],
+                 initial_manifest=None,
+                 base_path=None,
+                 add_conf_dirs=None):
 
         self.target_host = target_host
-        self.out_path = out_path
+
+        # FIXME: stopped: create base that does not require moving later
+        if base_path:
+            self.base_path = base_path
+        else:
+            self.base_path = tempfile.mkdtemp()
+
+        # FIXME: as well
+        self._init_cache_dir(None)
+
         self.exec_path = exec_path
+        self.custom_initial_manifest = initial_manifest
 
         self._add_conf_dirs = add_conf_dirs
 
         self._init_log()
         self._init_permissions()
         self._init_paths()
-        self._init_cache_dir(cache_dir)
         self._init_conf_dirs()
 
+
     @property
     def dist_conf_dir(self):
         return os.path.abspath(os.path.join(os.path.dirname(cdist.__file__), "conf"))
@@ -72,22 +88,23 @@ class Local(object):
 
     def _init_paths(self):
         # Depending on out_path
-        self.bin_path = os.path.join(self.out_path, "bin")
-        self.conf_path = os.path.join(self.out_path, "conf")
-        self.global_explorer_out_path = os.path.join(self.out_path, "explorer")
-        self.object_path = os.path.join(self.out_path, "object")
+        self.bin_path = os.path.join(self.base_path, "bin")
+        self.conf_path = os.path.join(self.base_path, "conf")
+        self.global_explorer_out_path = os.path.join(self.base_path, "explorer")
+        self.object_path = os.path.join(self.base_path, "object")
 
         # Depending on conf_path
         self.global_explorer_path = os.path.join(self.conf_path, "explorer")
         self.manifest_path = os.path.join(self.conf_path, "manifest")
+        self.initial_manifest = (self.custom_initial_manifest or
+                    os.path.join(self.manifest_path, "init"))
+
         self.type_path = os.path.join(self.conf_path, "type")
 
     def _init_conf_dirs(self):
         self.conf_dirs = []
 
-        # Comes with the distribution
-        system_conf_dir = os.path.abspath(os.path.join(os.path.dirname(cdist.__file__), "conf"))
-        self.conf_dirs.append(system_conf_dir)
+        self.conf_dirs.append(self.dist_conf_dir)
 
         # Is the default place for user created explorer, type and manifest
         if self.home_dir:
@@ -99,10 +116,21 @@ class Local(object):
             cdist_path_dirs.reverse()
             self.conf_dirs.extend(cdist_path_dirs)
 
-        # Add user supplied directories
+        # Add command line supplied directories
         if self._add_conf_dirs:
             self.conf_dirs.extend(self._add_conf_dirs)
 
+    def _init_directories(self):
+        self.mkdir(self.conf_path)
+        self.mkdir(self.global_explorer_out_path)
+        self.mkdir(self.bin_path)
+
+    def create_files_dirs(self):
+        self._init_directories()
+        self._create_conf_path_and_link_conf_dirs()
+        self._link_types_for_emulator()
+
+
     def _init_cache_dir(self, cache_dir):
         if cache_dir:
             self.cache_path = cache_dir
@@ -127,8 +155,8 @@ class Local(object):
         Return the output as a string.
 
         """
-        assert isinstance(command, (list, tuple)), "list or tuple argument expected, got: %s" % command
         self.log.debug("Local run: %s", command)
+        assert isinstance(command, (list, tuple)), "list or tuple argument expected, got: %s" % command
 
         if env is None:
             env = os.environ.copy()
@@ -155,17 +183,12 @@ class Local(object):
 
         return self.run(command, env, return_output)
 
-    def create_files_dirs(self):
-        self._create_context_dirs()
-        self._create_conf_path_and_link_conf_dirs()
-        self._link_types_for_emulator()
-
-    def _create_context_dirs(self):
-        self.mkdir(self.out_path)
-
-        self.mkdir(self.conf_path)
-        self.mkdir(self.global_explorer_out_path)
-        self.mkdir(self.bin_path)
+    def save_cache(self):
+        destination = os.path.join(self.cache_path, self.target_host)
+        self.log.debug("Saving " + self.base_path + " to " + destination)
+        if os.path.exists(destination):
+            shutil.rmtree(destination)
+        shutil.move(self.base_path, destination)
 
     def _create_conf_path_and_link_conf_dirs(self):
         # Link destination directories
@@ -184,7 +207,7 @@ class Local(object):
 
                 for entry in os.listdir(current_dir):
                     rel_entry_path = os.path.join(sub_dir, entry)
-                    src = os.path.join(conf_dir, sub_dir, entry)
+                    src = os.path.abspath(os.path.join(conf_dir, sub_dir, entry))
                     dst = os.path.join(self.conf_path, sub_dir, entry)
 
                     # Already exists? remove and link
